# frozen_string_literal: true

require 'import'
require 'ostruct'
require 'cht-lib-fabric/tasks/actionable'
require 'feature_config/actions/checks_tenant_crn_authz'
require 'feature_config/tasks/legacy_cache'

module FeatureConfig
  module Actions
    # Get data about enabled integration accounts
    #
    # Receives a `tenant` parameter which should be a string representation of
    # a tenant CRN
    #
    # Gets user-facing customer information from Redis cache which is
    # populated by `legacy_cache:prime` task. If not found in the cache, assumed not to exist.
    # Gets feature whitelist values (those not from the database) from environment variables
    # which are cached for the process lifetime.
    class CustomerExtendedSettingsEnabled
      include ::Platform::Tasks::Actionable
      include ChecksTenantCrnAuthz
      include Import[
        :inflector,
        :logger,
        'setting_def.customer_extended_setting',
        'stores.customer_extended_setting_store'
      ]

      def call
        crn = authorized_crn_from_context || return

        customer_id = crn[:customer]

        stored_customer_settings = customer_extended_setting_store.from_customer_id(customer_id)
        # Do one fallback check
        if stored_customer_settings.nil?
          ::FeatureConfig::Tasks::LegacyCache.new.do_customer_with_shard_lookup(customer_id)
          stored_customer_settings = customer_extended_setting_store.from_customer_id(customer_id)
        end
        # If no stored settings, the customer isn't found
        return unless stored_customer_settings

        customer_setting_h = stored_customer_settings.to_h

        context.customerExtendedSetting = (feature_on(customer_setting_h)).map { |f| inflector.upper_underscore(f.to_s) }
      end

      private

      def feature_on(hash)
        hash
          .keys
          .select { |k| hash[k] }
      end

    end
  end
end
