# frozen_string_literal: true

require 'import'
require 'cht-lib-fabric/tasks/actionable'
require 'active_record'
require 'cht-lib-fabric/crns/organization'
require 'feature_config/errors'
require 'feature_config/models/customer_info'
require 'feature_config/actions/checks_tenant_crn_authz'

module FeatureConfig
  module Actions
    # Get data about enabled integration accounts
    #
    # Receives a `tenant` parameter which should be a string representation of
    # a tenant CRN
    #
    # Gets this information from legacy MySQL on demand,
    # but caches for 24 hours.
    #
    # The existence of a customer is assumed based on whether we have a cache
    # of customer_info for that tenant. We get customer_info and shard
    # assignments in bulk for all customers using the `legacy_cache:prime`
    # rake task.
    #
    # Produces fields
    #   :hasAlertlogicAccount
    #   :hasAnsibleAccount
    #   :hasChefAccount
    #   :hasDatadogAccount
    #   :hasPuppetAccount
    #   :hasNewrelicAccount
    #   :defaultOrgId
    #   :defaultOrgIsFlexOrg
    class CustomerInfo
      include ::Platform::Tasks::Actionable
      include ChecksTenantCrnAuthz
      include Import[
        'cache.customer_info_redis',
        'cache.shard_assignment_redis',
        'legacy_mysql_activerecord',
        'logger',
        'settings',
        'stores.customer_setting_store'
      ]

      def call
        crn = authorized_crn_from_context || return

        customer_id = crn[:customer]

        stored_customer_settings = customer_setting_store.from_customer_id(customer_id)
        # If no stored settings, the customer isn't found
        unless stored_customer_settings
          logger.info "Customer not found in settings store"
          return
        end

        context.customerInfo = customer_info_values(customer_id)
      end

      private

      # Get customer info from cache, or
      # query from mysql and populate the cache
      #
      # @return [hash, nil]
      #   Camel case customer info values hash if found, else nil
      #   { "hasAlertlogicAccount" => true, ... }
      def customer_info_values(customer_id)
        cached = customer_info_redis.deserialize(customer_id)
        logger.debug "Cached value for #{customer_id}: #{cached}"
        return customer_info_from_raw(cached, customer_id) if cached

        raw = customer_info_from_sql(customer_id)

        logger.debug(
          "Saving hash for #{customer_id}: #{raw} (#{settings.customer_info_cache_seconds} seconds)"
        )
        customer_info_redis.serialize(
          customer_id,
          raw
        )
        # TODO: add TTL to cht-lib-fabric's redis serialize method
        customer_info_redis.expire(
          customer_id,
          settings.customer_info_cache_seconds
        )

        customer_info_from_raw(raw, customer_id)
      end

      def customer_info_from_raw(raw_hash, customer_id)
        return unless raw_hash && !raw_hash.empty?

        default_org_id_crn =
          begin
            Platform::CRNs::Organization.new(
              customer: customer_id,
              organization: raw_hash["default_org_id"]
            )
          rescue StandardError => ex
            logger.warn "Failed to construct CRN: #{ex}"
            nil
          end

        Models::CustomerInfo.from_string_hash(
          raw_hash.merge("default_org_id" => default_org_id_crn)
        )
      end

      # Connect to customer's MySQL shard and get raw customer info
      # hash
      #
      # @return [hash, nil]
      #   Snake case customer info values hash if found, else nil
      #   { "has_alertlogic_account" => 1, ... }
      def customer_info_from_sql(customer_id)
        shard_id = shard_assignment_redis.get(customer_id)
        unless shard_id && !shard_id.empty?
          logger.warn "No shard assignment found for customer_id #{customer_id}"
          return
        end

        legacy_mysql_activerecord.with_db("db#{shard_id}") do
          customer = Customer.find(customer_id)
          if customer
            {
              "has_alertlogic_account" => customer.alertlogic_accounts.exists?,
              "has_ansible_account" => customer.ansible_accounts.exists?,
              "has_chef_account" => customer.chef_accounts.exists?,
              "has_datadog_account" => customer.datadog_accounts.exists?,
              "has_newrelic_account" => customer.newrelic_accounts.exists?,
              "has_puppet_account" => customer.puppet_accounts.exists?,
              "default_org_id" => customer.default_organization.id,
              "default_org_is_flex_org" => customer.default_organization.flex_org,
              "has_benchmark_pricing_enabled" => customer.benchmark_price_enabled_at ? true : false,
              "product_entitlements" => ProductEntitlement.where(customer_id: customer_id, state: "Active").pluck(:cloud_type).map(&:upcase)
            }
          end
        end
      end

      # A customer's product entitlement is accessed via the product_entitlements  
      # table and can not be derived from just the Customer record.
      # We use this model/table to fetch the customer's license:
      #   CloudHealth, CloudHealth Hybrid, etc.
      class ProductEntitlement < ActiveRecord::Base
      end


      class Customer < ActiveRecord::Base
        %i[alertlogic ansible chef datadog integration puppet gcp_billing].each do |type|
          has_many "#{type}_accounts".to_sym
        end
        has_many :organizations

        def newrelic_accounts
          integration_accounts.where(model_type: 149)
        end

        def default_organization
          organizations.where(default_organization: true).first
        end
      end

      class Organization < ActiveRecord::Base
      end

      class AlertlogicAccount < ActiveRecord::Base
      end

      class AnsibleAccount < ActiveRecord::Base
      end

      class ChefAccount < ActiveRecord::Base
      end

      class DatadogAccount < ActiveRecord::Base
      end

      class IntegrationAccount < ActiveRecord::Base
      end

      class PuppetAccount < ActiveRecord::Base
      end

      class GcpBillingAccount < ActiveRecord::Base
        def has_big_query_fields?
          [big_query_project, big_query_dataset, big_query_table].all?
        end
        def has_big_query_invoice_enabled?
          has_big_query_fields? && (!big_query_skus_discovered_at.nil? && big_query_skus_discovered_at < Time.now)
        end
      end
    end
  end
end
