# frozen_string_literal: true

require 'import'
require 'ostruct'
require 'cht-lib-fabric/tasks/actionable'
require 'feature_config/actions/checks_tenant_crn_authz'
require 'feature_config/tasks/legacy_cache'
require 'json'

module FeatureConfig
  module Actions
    # Get data about enabled integration accounts
    #
    # Receives a `tenant` parameter which should be a string representation of
    # a tenant CRN
    #
    # Gets user-facing customer information from Redis cache which is
    # populated by `legacy_cache:prime` task. If not found in the cache, assumed not to exist.
    # Gets feature whitelist values (those not from the database) from environment variables
    # which are cached for the process lifetime.
    class CustomerSettingsEnabled
      include ::Platform::Tasks::Actionable
      include ChecksTenantCrnAuthz
      include Import[
        :inflector,
        :logger,
        'setting_def.feature_whitelist',
        'stores.customer_setting_store'
      ]

      def call
        crn = authorized_crn_from_context || return

        customer_id = crn[:customer]

        stored_customer_settings = customer_setting_store.from_customer_id(customer_id)
        # Do one fallback check to onboard customer on the fly
        # this should only happen for customers that were just onboarded as
        # new customers should get onboarded via legacy_cache:prime
        if stored_customer_settings.nil?
          logger.info "Falling back to lookup customer settings for #{customer_id}"
          ::FeatureConfig::Tasks::LegacyCache.new.do_customer_with_shard_lookup(customer_id)
          stored_customer_settings = customer_setting_store.from_customer_id(customer_id)
          logger.info "Falling back determined customer settings for #{customer_id} were #{JSON.pretty_generate(stored_customer_settings)}"
        end
        # If no stored settings, the customer isn't found
        return unless stored_customer_settings

        customer_setting_h = stored_customer_settings.to_h
        feature_whitelist_h = feature_whitelist_values(customer_id)

        cs = context.customerSetting = (
          feature_on(customer_setting_h) + feature_on(feature_whitelist_h)
        ).map { |f| inflector.upper_underscore(f.to_s) }
        # DEBUG
        if ENV['RACK_ENV'] == 'development' ||
           ENV['RACK_ENV'] == 'staging'
            logger.info JSON.pretty_generate(cs)
        end
        cs
      end

      private

      def feature_on(hash)
        hash
          .keys
          .select { |k| hash[k] }
      end

      # For a given tenant, get the feature whitelist configuration
      #
      # @return [hash]
      #   Camel case feature whitelist values
      #   { "forcedTls" => false, ... }
      def feature_whitelist_values(customer_id)
        feature_customer_map = fill_feature_customer_map
        feature_whitelist["available"].each_with_object({}) do |feature, acc|
          acc[inflector.camelize(feature)] =
            feature_customer_map[feature][customer_id] || false
        end
      end

      # A hash of the form:
      # { "forced_tls" => { 123 => true, 321 => true } }
      # where the first-level keys are features
      # and the second-level keys are customer IDs for which the feature is true.
      def fill_feature_customer_map
        @@feature_customer_map ||= Mutex.new.synchronize do
          feature_whitelist["available"].each_with_object({}) do |feature, acc|
            customer_list = ENV["#{feature.upcase}_CUSTOMER_LIST"].to_s.split(",").map(&:to_i)
            acc[feature] = customer_list.each_with_object({}) { |c, map| map[c] = true }
          end
        end

        @@feature_customer_map
      end
    end
  end
end
