# frozen_string_literal: true

require 'import'
require 'sidekiq'
require 'erb'

module FeatureConfig
  module Tasks
    class LegacyCache
      include ::Sidekiq::Worker
      include Import[
        'alert',
        'cache.customer_setting_redis',
        'cache.customer_extended_setting_redis',
        'cache.shard_assignment_redis',
        'logger',
        'legacy_mysql_activerecord',
        'setting_def.customer_setting',
        'setting_def.customer_extended_setting',
        'settings'
      ]

      def perform(_options)
        results = legacy_mysql_activerecord.with_db(:master) do |conn|
          logger.info "Getting all tenant info"
          conn.execute(
            "
              select
                customers.id,
                customers.feature_mask,
                customers.feature_mask_extended,
                shard_assignments.shard_id
              from
                customers
                join
                  shard_assignments
                on
                  shard_assignments.customer_id = customers.id
                join
                  data_stores
                on
                  data_stores.name = 'main' and
                  data_stores.id = shard_assignments.data_store_id
              where
                customers.is_deleted is false
            "
          ).to_a
        end

        logger.info "#{results.length} tenants received, pushing into Redis"

        before_keys = customer_setting_redis.keys
        before_extended_keys = customer_extended_setting_redis.keys
        results.each do |tenant|
          do_customer(tenant)
        end

        keys_to_delete = before_keys - results.map { |t| t['id'] }.compact.map(&:to_s)
        logger.info "#{keys_to_delete.length} tenants were in Redis but no longer in DB and will be deleted"
        keys_to_delete.each do |key|
          logger.debug "Deleting tenant #{key}"
          customer_setting_redis.del(key)
          shard_assignment_redis.del(key)
        end

        extended_keys_to_delete = before_extended_keys - results.map { |t| t['id'] }.compact.map(&:to_s)
        logger.info "#{extended_keys_to_delete.length} tenants were in Redis but no longer in DB and will be deleted"
        extended_keys_to_delete.each do |key|
          logger.debug "Deleting tenant #{key}"
          customer_extended_setting_redis.del(key)
        end
      end

      #
      # Processes a single tenant setting their supported feature flags
      # and shard assignment
      #
      def do_customer(tenant)
          # Get enabled customer features
          tf_by_key = get_customer_settings(tenant, false)

          logger.debug "Pushing settings for tenant #{tenant['id']} into redis"
          customer_setting_redis.serialize(tenant['id'], tf_by_key)
          shard_assignment_redis.set(tenant['id'], tenant['shard_id'])

          # Get enabled extended customer features
          tf_by_key_ext = get_customer_settings(tenant, true)

          logger.debug "Pushing extended settings for tenant #{tenant['id']} into redis"
          customer_extended_setting_redis.serialize(tenant['id'], tf_by_key_ext)
      end

      #
      # Process a single tenant and lookup their shard first
      #
      def do_customer_with_shard_lookup(customer_id)
        if customer_id
          legacy_mysql_activerecord.with_db(:master) do |conn|
            logger.info "Getting all tenant info"
            results = conn.execute(
              "
                select
                  customers.id,
                  customers.feature_mask,
                  customers.feature_mask_extended,
                  shard_assignments.shard_id
                from
                  customers
                  join
                    shard_assignments
                  on
                    shard_assignments.customer_id = customers.id
                  join
                    data_stores
                  on
                    data_stores.name = 'main' and
                    data_stores.id = shard_assignments.data_store_id
                where
                  customers.is_deleted is false and customers.id = #{customer_id}
              "
            ).to_a
            logger.debug "do_customer_with_shard_lookup results = [#{results}]"
            if (results.nil? || results.empty?) 
              logger.error("Unable to find customer #{customer_id} in database during onboarding")
            end
            do_customer(results[0]) unless results.nil? || results.empty?
          end
        end
      end

      #
      # Returns the customer settings based on FC bitmask. This is pulled
      # from the /config/setting/customer_settings.yml
      #
      def get_customer_settings(tenant, extended = false)
        logger.debug "FeatureConfig::Tasks::LegacyCache tenant #{tenant}"
        bitmask = extended ? tenant["feature_mask_extended"].to_i : tenant["feature_mask"].to_i
        settings = extended ? customer_extended_setting : customer_setting

        tf_by_key = settings.keys.each_with_object({}) do |key, acc|
          bit = settings[key]["bit"]
          acc[key] = !((bitmask || 0) & 2**bit).zero?
        end

        tf_by_key
      end

      def clear_all(_options)
        all_keys = customer_setting_redis.keys
        logger.info "Deleting #{all_keys.length} entries"
        all_keys.each do |key|
          customer_setting_redis.del(key)
        end
      end
    end
  end
end
